@@ -61,10 +61,10 @@
 /**
  * liblwgeom versions
  */
-#define LIBLWGEOM_VERSION "2.4.3"
+#define LIBLWGEOM_VERSION "2.5.0"
 #define LIBLWGEOM_VERSION_MAJOR "2"
-#define LIBLWGEOM_VERSION_MINOR "4"
-#define LIBLWGEOM_GEOS_VERSION "36"
+#define LIBLWGEOM_VERSION_MINOR "5"
+#define LIBLWGEOM_GEOS_VERSION "37"
 
 /** Return lwgeom version string (not to be freed) */
 const char* lwgeom_version(void);
@@ -187,6 +187,9 @@ typedef enum LWORD_T {
 #define SRID_UNKNOWN 0
 #define SRID_IS_UNKNOWN(x) ((int)x<=0)
 
+/* Invalid SRID value, for internal use */
+#define SRID_INVALID (999999 + 2)
+
 /*
 ** EPSG WGS84 geographics, OGC standard default SRS, better be in
 ** the SPATIAL_REF_SYS table!
@@ -281,7 +284,7 @@ BOX3D;
 
 /******************************************************************
 * GBOX structure.
-* We include the flags (information about dimensinality),
+* We include the flags (information about dimensionality),
 * so we don't have to constantly pass them
 * into functions that use the GBOX.
 */
@@ -361,14 +364,14 @@ POINT4D;
 */
 typedef struct
 {
-	/* Array of POINT 2D, 3D or 4D, possibly missaligned. */
+	/* Array of POINT 2D, 3D or 4D, possibly misaligned. */
 	uint8_t *serialized_pointlist;
 
 	/* Use FLAGS_* macros to handle */
 	uint8_t  flags;
 
-	int npoints;   /* how many points we are currently storing */
-	int maxpoints; /* how many points we have space for in serialized_pointlist */
+	uint32_t npoints;   /* how many points we are currently storing */
+	uint32_t maxpoints; /* how many points we have space for in serialized_pointlist */
 }
 POINTARRAY;
 
@@ -451,8 +454,8 @@ typedef struct
 	uint8_t flags;
 	GBOX *bbox;
 	int32_t srid;
-	int nrings;   /* how many rings we are currently storing */
-	int maxrings; /* how many rings we have space for in **rings */
+	uint32_t nrings;   /* how many rings we are currently storing */
+	uint32_t maxrings; /* how many rings we have space for in **rings */
 	POINTARRAY **rings; /* list of rings (list of points) */
 }
 LWPOLY; /* "light-weight polygon" */
@@ -464,8 +467,8 @@ typedef struct
 	uint8_t flags;
 	GBOX *bbox;
 	int32_t srid;
-	int ngeoms;   /* how many geometries we are currently storing */
-	int maxgeoms; /* how many geometries we have space for in **geoms */
+	uint32_t ngeoms;   /* how many geometries we are currently storing */
+	uint32_t maxgeoms; /* how many geometries we have space for in **geoms */
 	LWPOINT **geoms;
 }
 LWMPOINT;
@@ -477,8 +480,8 @@ typedef struct
 	uint8_t flags;
 	GBOX *bbox;
 	int32_t srid;
-	int ngeoms;   /* how many geometries we are currently storing */
-	int maxgeoms; /* how many geometries we have space for in **geoms */
+	uint32_t ngeoms;   /* how many geometries we are currently storing */
+	uint32_t maxgeoms; /* how many geometries we have space for in **geoms */
 	LWLINE **geoms;
 }
 LWMLINE;
@@ -490,8 +493,8 @@ typedef struct
 	uint8_t flags;
 	GBOX *bbox;
 	int32_t srid;
-	int ngeoms;   /* how many geometries we are currently storing */
-	int maxgeoms; /* how many geometries we have space for in **geoms */
+	uint32_t ngeoms;   /* how many geometries we are currently storing */
+	uint32_t maxgeoms; /* how many geometries we have space for in **geoms */
 	LWPOLY **geoms;
 }
 LWMPOLY;
@@ -503,8 +506,8 @@ typedef struct
 	uint8_t flags;
 	GBOX *bbox;
 	int32_t srid;
-	int ngeoms;   /* how many geometries we are currently storing */
-	int maxgeoms; /* how many geometries we have space for in **geoms */
+	uint32_t ngeoms;   /* how many geometries we are currently storing */
+	uint32_t maxgeoms; /* how many geometries we have space for in **geoms */
 	LWGEOM **geoms;
 }
 LWCOLLECTION;
@@ -516,8 +519,8 @@ typedef struct
 	uint8_t flags;
 	GBOX *bbox;
 	int32_t srid;
-	int ngeoms;   /* how many geometries we are currently storing */
-	int maxgeoms; /* how many geometries we have space for in **geoms */
+	uint32_t ngeoms;   /* how many geometries we are currently storing */
+	uint32_t maxgeoms; /* how many geometries we have space for in **geoms */
 	LWGEOM **geoms;
 }
 LWCOMPOUND; /* "light-weight compound line" */
@@ -529,8 +532,8 @@ typedef struct
 	uint8_t flags;
 	GBOX *bbox;
 	int32_t srid;
-	int nrings;    /* how many rings we are currently storing */
-	int maxrings;  /* how many rings we have space for in **rings */
+	uint32_t nrings;    /* how many rings we are currently storing */
+	uint32_t maxrings;  /* how many rings we have space for in **rings */
 	LWGEOM **rings; /* list of rings (list of points) */
 }
 LWCURVEPOLY; /* "light-weight polygon" */
@@ -542,8 +545,8 @@ typedef struct
 	uint8_t flags;
 	GBOX *bbox;
 	int32_t srid;
-	int ngeoms;   /* how many geometries we are currently storing */
-	int maxgeoms; /* how many geometries we have space for in **geoms */
+	uint32_t ngeoms;   /* how many geometries we are currently storing */
+	uint32_t maxgeoms; /* how many geometries we have space for in **geoms */
 	LWGEOM **geoms;
 }
 LWMCURVE;
@@ -555,8 +558,8 @@ typedef struct
 	uint8_t flags;
 	GBOX *bbox;
 	int32_t srid;
-	int ngeoms;   /* how many geometries we are currently storing */
-	int maxgeoms; /* how many geometries we have space for in **geoms */
+	uint32_t ngeoms;   /* how many geometries we are currently storing */
+	uint32_t maxgeoms; /* how many geometries we have space for in **geoms */
 	LWGEOM **geoms;
 }
 LWMSURFACE;
@@ -568,8 +571,8 @@ typedef struct
 	uint8_t flags;
 	GBOX *bbox;
 	int32_t srid;
-	int ngeoms;   /* how many geometries we are currently storing */
-	int maxgeoms; /* how many geometries we have space for in **geoms */
+	uint32_t ngeoms;   /* how many geometries we are currently storing */
+	uint32_t maxgeoms; /* how many geometries we have space for in **geoms */
 	LWPOLY **geoms;
 }
 LWPSURFACE;
@@ -581,8 +584,8 @@ typedef struct
 	uint8_t flags;
 	GBOX *bbox;
 	int32_t srid;
-	int ngeoms;   /* how many geometries we are currently storing */
-	int maxgeoms; /* how many geometries we have space for in **geoms */
+	uint32_t ngeoms;   /* how many geometries we are currently storing */
+	uint32_t maxgeoms; /* how many geometries we have space for in **geoms */
 	LWTRIANGLE **geoms;
 }
 LWTIN;
@@ -627,6 +630,7 @@ extern LWMPOLY* lwmpoly_add_lwpoly(LWMPOLY *mobj, const LWPOLY *obj);
 extern LWPSURFACE* lwpsurface_add_lwpoly(LWPSURFACE *mobj, const LWPOLY *obj);
 extern LWTIN* lwtin_add_lwtriangle(LWTIN *mobj, const LWTRIANGLE *obj);
 
+extern LWCOLLECTION* lwcollection_concat_in_place(LWCOLLECTION* col1, const LWCOLLECTION* col2);
 
 
 /***********************************************************************
@@ -735,6 +739,10 @@ extern void lwgeom_drop_srid(LWGEOM *lwgeom);
  */
 extern void lwgeom_add_bbox(LWGEOM *lwgeom);
 /**
+* Drop current bbox and calculate a fresh one.
+*/
+extern void lwgeom_refresh_bbox(LWGEOM *lwgeom);
+/**
 * Compute a box for geom and all sub-geometries, if not already computed
 */
 extern void lwgeom_add_bbox_deep(LWGEOM *lwgeom, GBOX *gbox);
@@ -764,14 +772,14 @@ extern int lwtype_is_collection(uint8_t type);
 /**
 * Given an lwtype number, what homogeneous collection can hold it?
 */
-extern int lwtype_get_collectiontype(uint8_t type);
+extern uint32_t lwtype_get_collectiontype(uint8_t type);
 
 /**
 * Return the type name string associated with a type number
 * (e.g. Point, LineString, Polygon)
 */
 extern const char *lwtype_name(uint8_t type);
-
+extern uint8_t lwtype_multitype(uint8_t type);
 
 /******************************************************************/
 
@@ -781,7 +789,7 @@ extern const char *lwtype_name(uint8_t type);
  * will set point's m=0 (or NaN) if pa is 3d or 2d
  * NOTE: point is a real POINT3D *not* a pointer
  */
-extern POINT4D getPoint4d(const POINTARRAY *pa, int n);
+extern POINT4D getPoint4d(const POINTARRAY *pa, uint32_t n);
 
 /*
  * copies a point from the point array into the parameter point
@@ -789,23 +797,23 @@ extern POINT4D getPoint4d(const POINTARRAY *pa, int n);
  * will set point's m=0 (or NaN) if pa is 3d or 2d
  * NOTE: this will modify the point4d pointed to by 'point'.
  */
-extern int getPoint4d_p(const POINTARRAY *pa, int n, POINT4D *point);
+extern int getPoint4d_p(const POINTARRAY *pa, uint32_t n, POINT4D *point);
 
 /*
  * copies a point from the point array into the parameter point
  * will set point's z=0 (or NaN) if pa is 2d
  * NOTE: point is a real POINT3D *not* a pointer
  */
-extern POINT3DZ getPoint3dz(const POINTARRAY *pa, int n);
-extern POINT3DM getPoint3dm(const POINTARRAY *pa, int n);
+extern POINT3DZ getPoint3dz(const POINTARRAY *pa, uint32_t n);
+extern POINT3DM getPoint3dm(const POINTARRAY *pa, uint32_t n);
 
 /*
  * copies a point from the point array into the parameter point
  * will set point's z=0 (or NaN) if pa is 2d
  * NOTE: this will modify the point3d pointed to by 'point'.
  */
-extern int getPoint3dz_p(const POINTARRAY *pa, int n, POINT3DZ *point);
-extern int getPoint3dm_p(const POINTARRAY *pa, int n, POINT3DM *point);
+extern int getPoint3dz_p(const POINTARRAY *pa, uint32_t n, POINT3DZ *point);
+extern int getPoint3dm_p(const POINTARRAY *pa, uint32_t n, POINT3DM *point);
 
 
 /*
@@ -813,14 +821,14 @@ extern int getPoint3dm_p(const POINTARRAY *pa, int n, POINT3DM *point);
  * z value (if present is not returned)
  * NOTE: point is a real POINT3D *not* a pointer
  */
-extern POINT2D getPoint2d(const POINTARRAY *pa, int n);
+extern POINT2D getPoint2d(const POINTARRAY *pa, uint32_t n);
 
 /*
  * copies a point from the point array into the parameter point
  * z value (if present is not returned)
  * NOTE: this will modify the point2d pointed to by 'point'.
  */
-extern int getPoint2d_p(const POINTARRAY *pa, int n, POINT2D *point);
+extern int getPoint2d_p(const POINTARRAY *pa, uint32_t n, POINT2D *point);
 
 /**
 * Returns a POINT2D pointer into the POINTARRAY serialized_ptlist,
@@ -828,7 +836,7 @@ extern int getPoint2d_p(const POINTARRAY *pa, int n, POINT2D *point);
 * and declared const because you aren't allowed to muck with the
 * values, only read them.
 */
-extern const POINT2D* getPoint2d_cp(const POINTARRAY *pa, int n);
+extern const POINT2D* getPoint2d_cp(const POINTARRAY *pa, uint32_t n);
 
 /**
 * Returns a POINT3DZ pointer into the POINTARRAY serialized_ptlist,
@@ -836,7 +844,7 @@ extern const POINT2D* getPoint2d_cp(const POINTARRAY *pa, int n);
 * and declared const because you aren't allowed to muck with the
 * values, only read them.
 */
-extern const POINT3DZ* getPoint3dz_cp(const POINTARRAY *pa, int n);
+extern const POINT3DZ* getPoint3dz_cp(const POINTARRAY *pa, uint32_t n);
 
 /**
 * Returns a POINT4D pointer into the POINTARRAY serialized_ptlist,
@@ -844,7 +852,7 @@ extern const POINT3DZ* getPoint3dz_cp(const POINTARRAY *pa, int n);
 * and declared const because you aren't allowed to muck with the
 * values, only read them.
 */
-extern const POINT4D* getPoint4d_cp(const POINTARRAY *pa, int n);
+extern const POINT4D* getPoint4d_cp(const POINTARRAY *pa, uint32_t n);
 
 /*
  * set point N to the given value
@@ -854,7 +862,7 @@ extern const POINT4D* getPoint4d_cp(const POINTARRAY *pa, int n);
  *
  * N must be a valid point index
  */
-extern void ptarray_set_point4d(POINTARRAY *pa, int n, const POINT4D *p4d);
+extern void ptarray_set_point4d(POINTARRAY *pa, uint32_t n, const POINT4D *p4d);
 
 /*
  * get a pointer to nth point of a POINTARRAY
@@ -865,13 +873,13 @@ extern void ptarray_set_point4d(POINTARRAY *pa, int n, const POINT4D *p4d);
  * WARNING: Don't cast this to a POINT !
  * it would not be reliable due to memory alignment constraints
  */
-extern uint8_t *getPoint_internal(const POINTARRAY *pa, int n);
+extern uint8_t *getPoint_internal(const POINTARRAY *pa, uint32_t n);
 
 /*
  * size of point represeneted in the POINTARRAY
  * 16 for 2d, 24 for 3d, 32 for 4d
  */
-extern int ptarray_point_size(const POINTARRAY *pa);
+extern size_t ptarray_point_size(const POINTARRAY *pa);
 
 
 /**
@@ -922,13 +930,13 @@ extern int ptarray_append_ptarray(POINTARRAY *pa1, POINTARRAY *pa2, double gap_t
 * Insert a point into an existing #POINTARRAY. Zero
 * is the index of the start of the array.
 */
-extern int ptarray_insert_point(POINTARRAY *pa, const POINT4D *p, int where);
+extern int ptarray_insert_point(POINTARRAY *pa, const POINT4D *p, uint32_t where);
 
 /**
 * Remove a point from an existing #POINTARRAY. Zero
 * is the index of the start of the array.
 */
-extern int ptarray_remove_point(POINTARRAY *pa, int where);
+extern int ptarray_remove_point(POINTARRAY *pa, uint32_t where);
 
 /**
  * @brief Add a point in a pointarray.
@@ -962,9 +970,6 @@ extern int ptarray_is_closed(const POINTARRAY *pa);
 extern int ptarray_is_closed_2d(const POINTARRAY *pa);
 extern int ptarray_is_closed_3d(const POINTARRAY *pa);
 extern int ptarray_is_closed_z(const POINTARRAY *pa);
-extern void ptarray_longitude_shift(POINTARRAY *pa);
-extern int ptarray_isccw(const POINTARRAY *pa);
-extern void ptarray_reverse(POINTARRAY *pa);
 extern POINTARRAY* ptarray_flip_coordinates(POINTARRAY *pa);
 
 /**
@@ -984,8 +989,9 @@ extern LWGEOM* lwgeom_force_3dz(const LWGEOM *geom);
 extern LWGEOM* lwgeom_force_3dm(const LWGEOM *geom);
 extern LWGEOM* lwgeom_force_4d(const LWGEOM *geom);
 
-extern LWGEOM* lwgeom_simplify(const LWGEOM *igeom, double dist, int preserve_collapsed);
 extern LWGEOM* lwgeom_set_effective_area(const LWGEOM *igeom, int set_area, double area);
+extern LWGEOM* lwgeom_chaikin(const LWGEOM *igeom, int n_iterations, int preserve_endpoint);
+extern LWGEOM* lwgeom_filter_m(LWGEOM *geom, double min, double max, int returnm);
 
 /*
  * Force to use SFS 1.1 geometry type
@@ -1015,7 +1021,12 @@ extern int lwpoint_getPoint4d_p(const LWPOINT *point, POINT4D *out);
 /**
  * Add a LWPOINT to an LWLINE
  */
-extern int lwline_add_lwpoint(LWLINE *line, LWPOINT *point, int where);
+extern int lwline_add_lwpoint(LWLINE *line, LWPOINT *point, uint32_t where);
+
+/**
+ * Interpolate one or more points along a line
+ */
+extern POINTARRAY* lwline_interpolate_points(const LWLINE *line, double length_fraction, char repeat);
 
 /******************************************************************
  * LWPOLY functions
@@ -1199,49 +1210,55 @@ extern double lwgeom_length(const LWGEOM *geom);
 extern double lwgeom_length_2d(const LWGEOM *geom);
 extern double lwgeom_perimeter(const LWGEOM *geom);
 extern double lwgeom_perimeter_2d(const LWGEOM *geom);
-extern void lwgeom_affine(LWGEOM *geom, const AFFINE *affine);
-extern void lwgeom_scale(LWGEOM *geom, const POINT4D *factors);
 extern int lwgeom_dimension(const LWGEOM *geom);
 
-extern LWPOINT* lwline_get_lwpoint(const LWLINE *line, int where);
-extern LWPOINT* lwcircstring_get_lwpoint(const LWCIRCSTRING *circ, int where);
+extern LWPOINT* lwline_get_lwpoint(const LWLINE *line, uint32_t where);
+extern LWPOINT* lwcircstring_get_lwpoint(const LWCIRCSTRING *circ, uint32_t where);
 
 extern LWPOINT* lwcompound_get_startpoint(const LWCOMPOUND *lwcmp);
 extern LWPOINT* lwcompound_get_endpoint(const LWCOMPOUND *lwcmp);
-extern LWPOINT* lwcompound_get_lwpoint(const LWCOMPOUND *lwcmp, int where);
-
+extern LWPOINT* lwcompound_get_lwpoint(const LWCOMPOUND *lwcmp, uint32_t where);
 
 extern double ptarray_length_2d(const POINTARRAY *pts);
-extern double ptarray_length(const POINTARRAY *pts);
-extern double ptarray_arc_length_2d(const POINTARRAY *pts);
-
-
 extern int pt_in_ring_2d(const POINT2D *p, const POINTARRAY *ring);
 extern int azimuth_pt_pt(const POINT2D *p1, const POINT2D *p2, double *ret);
 extern int lwpoint_inside_circle(const LWPOINT *p, double cx, double cy, double rad);
-extern void lwgeom_reverse(LWGEOM *lwgeom);
-extern void lwline_reverse(LWLINE *line);
-extern void lwpoly_reverse(LWPOLY *poly);
-extern void lwtriangle_reverse(LWTRIANGLE *triangle);
+
+extern LWGEOM* lwgeom_reverse(const LWGEOM *lwgeom);
 extern char* lwgeom_summary(const LWGEOM *lwgeom, int offset);
 extern char* lwpoint_to_latlon(const LWPOINT *p, const char *format);
 extern int lwgeom_startpoint(const LWGEOM* lwgeom, POINT4D* pt);
 
+extern void interpolate_point4d(const POINT4D *A, const POINT4D *B, POINT4D *I, double F);
+
 /**
 * Ensure the outer ring is clockwise oriented and all inner rings
 * are counter-clockwise.
 */
-extern void lwgeom_force_clockwise(LWGEOM *lwgeom);
-extern void lwpoly_force_clockwise(LWPOLY *poly);
-extern void lwtriangle_force_clockwise(LWTRIANGLE *triangle);
-
 extern int lwgeom_is_clockwise(LWGEOM *lwgeom);
-extern int lwpoly_is_clockwise(LWPOLY *poly);
-extern int lwtriangle_is_clockwise(LWTRIANGLE *triangle);
 
-extern void interpolate_point4d(POINT4D *A, POINT4D *B, POINT4D *I, double F);
 
-void lwgeom_longitude_shift(LWGEOM *lwgeom);
+extern LWGEOM* lwgeom_simplify(const LWGEOM *igeom, double dist, int preserve_collapsed);
+extern LWGEOM* lwgeom_remove_repeated_points(const LWGEOM *in, double tolerance);
+
+/****************************************************************
+* READ/WRITE FUNCTIONS
+*
+* Coordinate writing functions, which will alter the coordinates
+* and potentially the structure of the input geometry. When
+* called from within PostGIS, the LWGEOM argument should be built
+* on top of a gserialized copy, created using
+* PG_GETARG_GSERIALIZED_P_COPY()
+****************************************************************/
+
+extern void lwgeom_reverse_in_place(LWGEOM *lwgeom);
+extern void lwgeom_force_clockwise(LWGEOM *lwgeom);
+extern void lwgeom_longitude_shift(LWGEOM *lwgeom);
+extern void lwgeom_simplify_in_place(LWGEOM *igeom, double dist, int preserve_collapsed);
+extern void lwgeom_affine(LWGEOM *geom, const AFFINE *affine);
+extern void lwgeom_scale(LWGEOM *geom, const POINT4D *factors);
+extern void lwgeom_remove_repeated_points_in_place(LWGEOM *in, double tolerance);
+
 
 /**
  * @brief wrap geometry on given cut x value
@@ -1272,13 +1289,13 @@ extern int lwgeom_needs_bbox(const LWGEOM *geom);
 /**
 * Count the total number of vertices in any #LWGEOM.
 */
-extern int lwgeom_count_vertices(const LWGEOM *geom);
+extern uint32_t lwgeom_count_vertices(const LWGEOM *geom);
 
 /**
 * Count the total number of rings in any #LWGEOM. Multipolygons
 * and other collections get counted, not the same as OGC st_numrings.
 */
-extern int lwgeom_count_rings(const LWGEOM *geom);
+extern uint32_t lwgeom_count_rings(const LWGEOM *geom);
 
 /**
 * Return true or false depending on whether a geometry has
@@ -1301,17 +1318,10 @@ extern int lwgeom_is_closed(const LWGEOM *geom);
 /**
 * Return the dimensionality (relating to point/line/poly) of an lwgeom
 */
-extern int lwgeom_dimensionality(LWGEOM *geom);
+extern int lwgeom_dimensionality(const LWGEOM *geom);
 
 /* Is lwgeom1 geometrically equal to lwgeom2 ? */
-char lwgeom_same(const LWGEOM *lwgeom1, const LWGEOM *lwgeom2);
-char ptarray_same(const POINTARRAY *pa1, const POINTARRAY *pa2);
-char lwpoint_same(const LWPOINT *p1, const LWPOINT *p2);
-char lwline_same(const LWLINE *p1, const LWLINE *p2);
-char lwpoly_same(const LWPOLY *p1, const LWPOLY *p2);
-char lwtriangle_same(const LWTRIANGLE *p1, const LWTRIANGLE *p2);
-char lwcollection_same(const LWCOLLECTION *p1, const LWCOLLECTION *p2);
-char lwcircstring_same(const LWCIRCSTRING *p1, const LWCIRCSTRING *p2);
+extern char lwgeom_same(const LWGEOM *lwgeom1, const LWGEOM *lwgeom2);
 
 
 /**
@@ -1327,10 +1337,7 @@ extern LWGEOM *lwgeom_clone(const LWGEOM *lwgeom);
 * Deep clone an LWGEOM, everything is copied
 */
 extern LWGEOM *lwgeom_clone_deep(const LWGEOM *lwgeom);
-
-/* TODO Move to Internal */
-LWPOINT *lwpoint_clone(const LWPOINT *lwgeom);
-POINTARRAY *ptarray_clone_deep(const POINTARRAY *ptarray);
+extern POINTARRAY *ptarray_clone_deep(const POINTARRAY *ptarray);
 
 
 /*
@@ -1477,18 +1484,18 @@ extern int lwline_is_trajectory(const LWLINE *geom);
  * Ensure every segment is at most 'dist' long.
  * Returned LWGEOM might is unchanged if a POINT.
  */
-extern LWGEOM *lwgeom_segmentize2d(LWGEOM *line, double dist);
+extern LWGEOM *lwgeom_segmentize2d(const LWGEOM *line, double dist);
 extern POINTARRAY *ptarray_segmentize2d(const POINTARRAY *ipa, double dist);
-extern LWLINE *lwline_segmentize2d(LWLINE *line, double dist);
-extern LWPOLY *lwpoly_segmentize2d(LWPOLY *line, double dist);
-extern LWCOLLECTION *lwcollection_segmentize2d(LWCOLLECTION *coll, double dist);
+extern LWLINE *lwline_segmentize2d(const LWLINE *line, double dist);
+extern LWPOLY *lwpoly_segmentize2d(const LWPOLY *line, double dist);
+extern LWCOLLECTION *lwcollection_segmentize2d(const LWCOLLECTION *coll, double dist);
 
 /*
  * Point density functions
  */
-extern LWMPOINT *lwpoly_to_points(const LWPOLY *poly, int npoints);
-extern LWMPOINT *lwmpoly_to_points(const LWMPOLY *mpoly, int npoints);
-extern LWMPOINT *lwgeom_to_points(const LWGEOM *lwgeom, int npoints);
+extern LWMPOINT *lwpoly_to_points(const LWPOLY *poly, uint32_t npoints);
+extern LWMPOINT *lwmpoly_to_points(const LWMPOLY *mpoly, uint32_t npoints);
+extern LWMPOINT *lwgeom_to_points(const LWGEOM *lwgeom, uint32_t npoints);
 
 /*
  * Geometric median
@@ -1659,13 +1666,6 @@ extern void lwboundingcircle_destroy(LWBOUNDINGCIRCLE* c);
  */
 extern LWBOUNDINGCIRCLE* lwgeom_calculate_mbc(const LWGEOM* g);
 
-/**
-* Remove repeated points!
-*/
-extern LWGEOM* lwgeom_remove_repeated_points(const LWGEOM *in, double tolerance);
-
-extern char lwtriangle_is_repeated_points(LWTRIANGLE *triangle);
-
 /**
  * Swap ordinate values in every vertex of the geometry.
  *
@@ -1680,14 +1680,6 @@ extern char lwtriangle_is_repeated_points(LWTRIANGLE *triangle);
  */
 extern void lwgeom_swap_ordinates(LWGEOM *in, LWORD o1, LWORD o2);
 
-/**
-* Reverse the X and Y coordinate order. Useful for geometries in lat/lon order
-* than need to be converted to lon/lat order.
-*
-* NOTE: uses lwgeom_swap_ordinates internally,
-*       kept for backward compatibility
-*/
-extern LWGEOM* lwgeom_flip_coordinates(LWGEOM *in);
 
 struct LWPOINTITERATOR;
 typedef struct LWPOINTITERATOR LWPOINTITERATOR;
@@ -1821,7 +1813,7 @@ extern int lwgeom_calculate_gbox(const LWGEOM *lwgeom, GBOX *gbox);
 * New function to read doubles directly from the double* coordinate array
 * of an aligned lwgeom #POINTARRAY (built by de-serializing a #GSERIALIZED).
 */
-extern int getPoint2d_p_ro(const POINTARRAY *pa, int n, POINT2D **point);
+extern int getPoint2d_p_ro(const POINTARRAY *pa, uint32_t n, POINT2D **point);
 
 /**
 * Calculate geodetic (x/y/z) box and add values to gbox. Return #LW_SUCCESS on success.
@@ -2019,8 +2011,8 @@ extern int gserialized_get_gbox_p(const GSERIALIZED *g, GBOX *gbox);
 typedef struct struct_lwgeom_parser_result
 {
 	const char *wkinput;        /* Copy of pointer to input WKT/WKB */
-	uint8_t *serialized_lwgeom;   /* Pointer to serialized LWGEOM */
-	int size;                   /* Size of serialized LWGEOM in bytes */
+	uint8_t *serialized_lwgeom; /* Pointer to serialized LWGEOM */
+	size_t size;                /* Size of serialized LWGEOM in bytes */
 	LWGEOM *geom;               /* Pointer to LWGEOM struct */
 	const char *message;        /* Error/warning message */
 	int errcode;                /* Error/warning number */
@@ -2052,7 +2044,7 @@ typedef struct struct_lwgeom_unparser_result
 {
 	uint8_t *serialized_lwgeom;	/* Copy of pointer to input serialized LWGEOM */
 	char *wkoutput;			/* Pointer to WKT or WKB output */
-	int size;			/* Size of serialized LWGEOM in bytes */
+	size_t size;			/* Size of serialized LWGEOM in bytes */
 	const char *message;		/* Error/warning message */
 	int errlocation;		/* Location of error */
 }
@@ -2143,7 +2135,7 @@ extern LWGEOM* lwgeom_from_hexwkb(const char *hexwkb, const char check);
 
 extern uint8_t*  bytes_from_hexbytes(const char *hexbuf, size_t hexsize);
 
-extern char*   hexbytes_from_bytes(uint8_t *bytes, size_t size);
+extern char*   hexbytes_from_bytes(const uint8_t *bytes, size_t size);
 
 /*
 * WKT detailed parsing support
@@ -2169,7 +2161,7 @@ extern char *lwmessage_truncate(char *str, int startpos, int endpos, int maxleng
  * @param check parser check flags, see LW_PARSER_CHECK_* macros
  * @param size parser check flags, see LW_PARSER_CHECK_* macros
  */
-extern LWGEOM* lwgeom_from_twkb(uint8_t *twkb, size_t twkb_size, char check);
+extern LWGEOM* lwgeom_from_twkb(const uint8_t *twkb, size_t twkb_size, char check);
 
 /**
  * @param geom input geometry
@@ -2180,6 +2172,20 @@ extern uint8_t* lwgeom_to_twkb(const LWGEOM *geom, uint8_t variant, int8_t preci
 
 extern uint8_t* lwgeom_to_twkb_with_idlist(const LWGEOM *geom, int64_t *idlist, uint8_t variant, int8_t precision_xy, int8_t precision_z, int8_t precision_m, size_t *twkb_size);
 
+/**
+ * Trim the bits of an LWGEOM in place, to optimize it for compression.
+ * Sets all bits to zero that are not required to maintain a specified
+ * number of digits after the decimal point. Negative precision values
+ * indicate digits before the decimal point do not need to be preserved.
+ *
+ * @param geom input geometry
+ * @param prec_x precision (digits after decimal point) in x dimension
+ * @param prec_y precision (digits after decimal point) in y dimension
+ * @param prec_z precision (digits after decimal point) in z dimension
+ * @param prec_m precision (digits after decimal point) in m dimension
+ */
+extern void lwgeom_trim_bits_in_place(LWGEOM *geom, int32_t prec_x, int32_t prec_y, int32_t prec_z, int32_t prec_m);
+
 /*******************************************************************************
  * SQLMM internal functions
  ******************************************************************************/
@@ -2269,12 +2275,13 @@ LWGEOM *lwgeom_normalize(const LWGEOM *geom);
 LWGEOM *lwgeom_intersection(const LWGEOM *geom1, const LWGEOM *geom2);
 LWGEOM *lwgeom_difference(const LWGEOM *geom1, const LWGEOM *geom2);
 LWGEOM *lwgeom_symdifference(const LWGEOM* geom1, const LWGEOM* geom2);
+LWGEOM *lwgeom_pointonsurface(const LWGEOM* geom);
 LWGEOM *lwgeom_centroid(const LWGEOM* geom);
 LWGEOM *lwgeom_union(const LWGEOM *geom1, const LWGEOM *geom2);
 LWGEOM *lwgeom_linemerge(const LWGEOM *geom1);
 LWGEOM *lwgeom_unaryunion(const LWGEOM *geom1);
 LWGEOM *lwgeom_clip_by_rect(const LWGEOM *geom1, double x0, double y0, double x1, double y1);
-LWCOLLECTION *lwgeom_subdivide(const LWGEOM *geom, int maxvertices);
+LWCOLLECTION *lwgeom_subdivide(const LWGEOM *geom, uint32_t maxvertices);
 
 /**
  * Snap vertices and segments of a geometry to another using a given tolerance.
@@ -2301,16 +2308,15 @@ LWGEOM* lwgeom_sharedpaths(const LWGEOM* geom1, const LWGEOM* geom2);
 /*
  * An offset curve against the input line.
  *
- * @param lwline a lineal geometry
+ * @param geom a lineal geometry or collection of them
  * @param size offset distance. Offset left if negative and right if positive
  * @param quadsegs number of quadrature segments in curves (try 8)
  * @param joinStyle (1 = round, 2 = mitre, 3 = bevel)
  * @param mitreLimit (try 5.0)
  * @return derived geometry (linestring or multilinestring)
  *
- * Requires GEOS-3.2.0+
  */
-LWGEOM* lwgeom_offsetcurve(const LWLINE *lwline, double size, int quadsegs, int joinStyle, double mitreLimit);
+LWGEOM* lwgeom_offsetcurve(const LWGEOM *geom, double size, int quadsegs, int joinStyle, double mitreLimit);
 
 /*
  * Return true if the input geometry is "simple" as per OGC defn.
@@ -2338,9 +2344,8 @@ projPJ lwproj_from_string(const char* txt);
  * @param inpj the input (or current, or source) projection
  * @param outpj the output (or destination) projection
  */
-int lwgeom_transform(LWGEOM *geom, projPJ inpj, projPJ outpj) ;
-int ptarray_transform(POINTARRAY *geom, projPJ inpj, projPJ outpj) ;
-int point4d_transform(POINT4D *pt, projPJ srcpj, projPJ dstpj) ;
+int lwgeom_transform(LWGEOM *geom, projPJ inpj, projPJ outpj);
+int ptarray_transform(POINTARRAY *pa, projPJ inpj, projPJ outpj);
 
 
 /*******************************************************************************
@@ -2356,12 +2361,8 @@ int point4d_transform(POINT4D *pt, projPJ srcpj, projPJ dstpj) ;
  */
 LWGEOM* lwgeom_buildarea(const LWGEOM *geom) ;
 
-
 /**
  * Attempts to make an invalid geometries valid w/out losing points.
- *
- * NOTE: this is only available when liblwgeom is built against
- *       GEOS 3.3.0 or higher
  */
 LWGEOM* lwgeom_make_valid(LWGEOM* geom);
 
@@ -2377,8 +2378,6 @@ LWGEOM* lwgeom_split(const LWGEOM* lwgeom_in, const LWGEOM* blade_in);
 /*
  * Fully node a set of linestrings, using the least nodes preserving
  * all the input ones.
- *
- * Requires GEOS-3.3.0 or higher
  */
 LWGEOM* lwgeom_node(const LWGEOM* lwgeom_in);
 
@@ -2391,7 +2390,7 @@ LWGEOM* lwgeom_node(const LWGEOM* lwgeom_in);
  * @param edgeOnly if non-zero the result will be a MULTILINESTRING,
  *                 otherwise it'll be a COLLECTION of polygons.
  */
-LWGEOM* lwgeom_delaunay_triangulation(const LWGEOM *geom, double tolerance, int edgeOnly);
+LWGEOM* lwgeom_delaunay_triangulation(const LWGEOM *geom, double tolerance, int32_t edgeOnly);
 
 /**
  * Take vertices of a geometry and build the Voronoi diagram
@@ -2401,8 +2400,6 @@ LWGEOM* lwgeom_delaunay_triangulation(const LWGEOM *geom, double tolerance, int
  * @param tolerance an optional snapping tolerance for improved robustness
  * @param output_edges if non-zero the result will be a MULTILINESTRING,
  *                 otherwise it'll be a COLLECTION of polygons.
- *
- * Requires GEOS-3.5.0 or higher
  */
 LWGEOM* lwgeom_voronoi_diagram(const LWGEOM* g, const GBOX* env, double tolerance, int output_edges);
 
@@ -2414,8 +2411,7 @@ LWGEOM* lwgeom_voronoi_diagram(const LWGEOM* g, const GBOX* env, double toleranc
 * @param ngeoms the number of elements in the array
 * @param k the number of clusters to calculate
 */
-int * lwgeom_cluster_2d_kmeans(const LWGEOM **geoms, int ngeoms, int k);
-
+int * lwgeom_cluster_2d_kmeans(const LWGEOM **geoms, uint32_t ngeoms, uint32_t k);
 
 #endif /* !defined _LIBLWGEOM_H  */
 
